transformpath

package
v0.1.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 6, 2025 License: MIT Imports: 1 Imported by: 0

Documentation

Overview

Package transformpath carries collection paths across the public transform package boundary without exposing an intermediate error type to users.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Index

func Index(err error, index int) error

func Key

func Key(err error, key any) error

Types

type Error

type Error struct {
	Segments []Segment
	Err      error
}

func (*Error) Error

func (e *Error) Error() string

func (*Error) Unwrap

func (e *Error) Unwrap() error

type Segment

type Segment struct {
	Key     any
	Index   int
	IsIndex bool
}

Jump to