package
Version:
v0.1.0
Opens a new window with list of versions in this module.
Published: Apr 6, 2025
License: MIT
Opens a new window with license information.
Imports: 1
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
Package transformpath carries collection paths across the public transform
package boundary without exposing an intermediate error type to users.
type Segment struct {
Key any
Index int
IsIndex bool
}
Source Files
¶
Click to show internal directories.
Click to hide internal directories.